#ifndef __PT_HELPERS_H
#define __PT_HELPERS_H

#include <stdint.h>
#include <stdbool.h>
#include <math.h>
#include "pt_header.h"

// rounded constants to fit in floats
#define M_PI_F  3.1415927f
#define M_2PI_F 6.2831853f

// fast 32-bit -> 16-bit clamp
#define CLAMP16(i) if ((int16_t)(i) != i) i = 0x7FFF ^ (i >> 31)

// fast 16-bit -> 8-bit clamp
#define CLAMP8(i) if ((int8_t)(i) != i) i = 0x7F ^ (i >> 15)

#define SWAP16(value) \
( \
	(((uint16_t)((value) & 0x00FF)) << 8) | \
	(((uint16_t)((value) & 0xFF00)) >> 8)   \
)

#define SWAP32(value) \
( \
	(((uint32_t)((value) & 0x000000FF)) << 24) | \
	(((uint32_t)((value) & 0x0000FF00)) <<  8) | \
	(((uint32_t)((value) & 0x00FF0000)) >>  8) | \
	(((uint32_t)((value) & 0xFF000000)) >> 24)   \
)

// arithmetic shift right on signed variables macros (works properly on all platforms, though not branchless...)
#if defined (__APPLE__) || defined (_WIN32)
#define  SAR8(x, n) ((x) >> (n))
#define SAR16(x, n) ((x) >> (n))
#define SAR32_1(x)  ((x) >>  1 )
#else
#define  SAR8(x, n) (int8_t)(((x) >= 0) ? ((x) >> (n)) : (((1 <<  8) - (1 << (8  - (n)))) | ((x) >> (n))))
#define SAR16(x, n) (int16_t)(((x) >= 0) ? ((x) >> (n)) : (((1 << 16) - (1 << (16 - (n)))) | ((x) >> (n))))
#define SAR32_1(x)  (int32_t)(((x) >= 0) ? ((x) >>  1 ) : (0x80000000L | ((x) >> 1)))
#endif

#define SGN(x) (((x) >= 0) ? 1 : -1)
#define ABS(a) (((a) < 0) ? -(a) : (a))
#define MIN(a, b) (((a) < (b)) ? (a) : (b))
#define MAX(a, b) (((a) > (b)) ? (a) : (b))
#define CLAMP(x, low, high) (((x) > (high)) ? (high) : (((x) < (low)) ? (low) : (x)))
#define LERP(x, y, z) ((x) + ((y) - (x)) * (z))

#define R12(x) (((x) >> 8) & 15)
#define G12(x) (((x) >> 4) & 15)
#define B12(x) (((x) >> 0) & 15)

#define R12_to_R24(x) (R12(x) * 17)
#define G12_to_G24(x) (G12(x) * 17)
#define B12_to_B24(x) (B12(x) * 17)

#define RGB12_to_RGB24(x) ((R12_to_R24(x) << 16) | (G12_to_G24(x) << 8) | (B12_to_B24(x)))

#define R24(x) (((x) >> 16) & 255)
#define G24(x) (((x) >>  8) & 255)
#define B24(x) (((x) >>  0) & 255)

#define RGB24(r, g, b) (((r) << 16) | ((g) << 8) | (b))

void showErrorMsgBox(const char *fmt, ...);
#ifndef _WIN32
bool changePathToHome(void);
#else
// Windows usleep() implementation
void usleep(uint32_t usec);
void setupWin32Usleep(void);
void freeWin32Usleep(void);
#endif
bool sampleNameIsEmpty(char *name);
bool moduleNameIsEmpty(char *name);
void updateWindowTitle(bool modified);
void recalcChordLength(void);

#endif
